{% extends "layout.html" %}
{% block title %}- My seeded torrents{% endblock %}
{% block body %}
    <table class="table is-bordered is-striped is-narrow is-fullwidth is-hoverable is-size-7">
        <thead>
        <tr>
            <th>Name</th>
            <th>Link</th>
            <th>Season</th>
            <th>Tools</th>
        </tr>
        </thead>
        <tbody>

        {% for title in titles.values() %}
            {% for link in title %}
                <tr>
                    {% if not loop.index0 %}
                        <td rowspan="{{ title|length }}">
                            {{ link.title.name }}
                        </td>
                    {% endif %}

                    <td class="{{ link|colorify }}">
                        {{ link.vf|flagify }}
                        {{ link.link|urlize(30, target='_blank') }}
                    </td>

                    <td>
                        {{ link.season }}
                    </td>

                    {% if not loop.index0 %}
                        <td rowspan="{{ title|length }}">
                            <a href="{{ url_for('search', q=link.title.keyword) }}" target="_blank">
                                <i class="fa fa-search"></i>
                            </a>
                        </td>
                    {% endif %}
                </tr>
            {% endfor %}
        {% endfor %}

        </tbody>
    </table>
{% endblock %}
